// Deps
const _       = require('lodash');
const express = require('express');
const path    = require('path');
const moment  = require('moment');

// Routes
const mainRouter = express.Router();

// Globals
var db;
var options;
var adminPath;

/**
 * Creates renders and returns html
 *
 * @param {object} res
 * @param {string} templateFilename
 * @param {object} params
 */
const render = function (response, params) {
  "use strict";

  params = params || {};

  _.set(params, 'moment', moment);
  _.set(params, 'models', Object.keys(db.models));
  _.set(params, 'model', response.model);
  _.set(params, 'modelname', response.modelname);
  _.set(params, 'schema', response.schema);
  _.set(params, 'adminPath', adminPath);
  _.set(params, 'title', _.get(options, 'title', 'SeqAdmin'));
  _.set(params, 'linkField', _.get(response, 'actionOptions.link'));

  _.set(params, 'template_order', function (key, currentOrder) {
    var keyOrder = currentOrder[0]==key && currentOrder[1]=='ASC' ? 'DESC' : 'ASC';
    var html = '<a href="?order='+key+','+keyOrder+'"' + key + '>'+key+'</a>';
    return html;
  });

  return response.render(path.join(__dirname, './views/' + response.action + '.html'), params);
};

/**
 * List view
 *
 * @param {object} req
 * @param {object} res
 */
const listCtrl = function (req, res) {
  "use strict";

  var queryOptions = _.get(res.actionOptions, 'query', {});
  if (req.query.order) {
    queryOptions.order = [req.query.order.split(',')];
  }

  res.model
  .findAll(queryOptions)
  .then(function (results) {
    return render(res, {
      results: results,
      currentOrder: _.get(queryOptions, 'order[0]'),
    });
  });
}

/**
 * Edit view
 *
 * @param {object} req
 * @param {object} res
 */
const editCtrl = function (req, res) {
  res.model
  .findOne({
    where: {
      id: res.id
    }
  })
  .then(function (result) {
    return render(res, {
      result: result
    });
  });
}

/**
 * Decides routes and inits params
 *
 * @param {object} req
 * @param {object} res
 */
mainRouter.use(function (req, res) {
  _.set(res, 'modelname', _.get(req.path.split('/'), '[1]'));
  _.set(res, 'model', _.get(db.models, res.modelname));

  if (!res.model) {
    res.action = 'home';
    return render(res);
  }

  _.set(res, 'id', _.get(req.path.split('/'), '[2]'));
  _.set(res, 'action', res.id ? (_.get(req.path.split('/'), '[3]') || 'edit') : 'list');
  _.set(res, 'actionOptions', _.get(options, 'models.' + res.modelname + '.' + res.action));

  res.model.describe().then(function(schema) {
    _.set(res, 'schema', _.omit(schema, _.get(res.actionOptions, 'hide')));

    switch (res.action) {
      case 'list': return listCtrl(req, res); break;
      case 'edit': return editCtrl(req, res); break;
    }
  });
});

/**
 * Export
 */
module.exports = function (app, dbData, optionsContainer) {
  adminPath = _.get(optionsContainer, 'path') || '/admin';
  db = dbData;
  options = optionsContainer || {};

  app.use(adminPath, mainRouter);
};